/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2007 Hiroyuki Ikezoe All rights reserved.
 * Copyright (c) 2007 Kouhei Sutou All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of authors nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <assert.h>
#include <string.h>
#include <glib.h>
#include <pango/pango.h>

#include "PangoTextRenderer.h"
#include "PangoTextRendererFactory.h"

PangoTextRenderer::PangoTextRenderer(PangoTextRendererFactory* aowner,
                                     PangoNSFont *afont)
    : m_owner(aowner),
      m_font(afont),
      m_xoffset(0),
      m_yoffset(0)
{
    m_font->retain();
    ascent = m_font->ascent;
    descent = m_font->descent;
    lineSpacing = m_font->lineSpacing;
    xHeight = m_font->xHeight;
}

PangoTextRenderer::~PangoTextRenderer()
{
    m_font->release();
}

void PangoTextRenderer::setContext(CGContextRef acontext)
{
    assert(acontext);
    m_context = acontext;
    m_xoffset = m_yoffset = 0;
    m_context->realTranslate(&m_xoffset, &m_yoffset);
}

float PangoTextRenderer::floatWidthForRun(const WebCoreTextRun *run,
                                          const WebCoreTextStyle *style,
                                          float *widths)
{
    assert(run);
    assert(style);
    assert(style->families);

    if (run->length == 0)
        return 0.0f;

    PangoLayout *layout = getPangoLayout();
    PangoContext *pango_context;
    PangoFontMap *map;

    map = pango_cairo_font_map_get_default();

    setPangoAttributes(style, layout);

    pango_context = pango_layout_get_context(layout);

    gchar *utf8 = convertUniCharToUTF8(run->characters, 0, run->to);
    pango_layout_set_text(layout, utf8, -1);

    int pixel_width;
    pango_layout_get_pixel_size(layout, &pixel_width, 0);
    float width = (float) pixel_width;
    g_free(utf8);
    g_object_unref(layout);

    return width + style->padding;
}

void PangoTextRenderer::drawRun(const WebCoreTextRun *run, const WebCoreTextStyle *style, int x, int y)
{
    assert(run);
    assert(style);
    assert(style->families);

    if (run->length == 0)
        return;

    x -=  m_xoffset;
    y -=  m_yoffset;
	
    PangoLayout *layout = getPangoLayoutForGdkDrawable(m_context->realDrawable());
    setPangoAttributes(style, layout);

// FIXME! use these properties.
//    style.visuallyOrdered =  visuallyOrdered;
//    style.wordSpacing = wordSpacing;
//    style.padding = toAdd;

    gchar *utf8 = convertUniCharToUTF8(run->characters, 0, run->to);
    if (run->from > 0) {
        gchar *str_left = g_strdup(utf8);
        str_left = g_utf8_strncpy(str_left, utf8, run->from);
        PangoLayout *dummy_layout;
        int width;
        dummy_layout = pango_layout_copy(layout);
        pango_layout_set_text(dummy_layout, str_left, -1);
        pango_layout_get_pixel_size(dummy_layout, &width, 0);
        x += width;
        g_object_unref(dummy_layout);
        g_free(str_left);
    }

    gchar *str_from = g_utf8_offset_to_pointer(utf8, run->from);
    pango_layout_context_changed(layout);
    pango_layout_set_text(layout, str_from, -1);

    GdkGC *gc = gdk_gc_new(m_context->realDrawable());
    gdk_draw_layout(m_context->realDrawable(), gc, x, y - ascent, layout);
    g_object_unref(gc);
    g_object_unref(layout);
    g_free(utf8);
}

void PangoTextRenderer::drawHighlightForRun(const WebCoreTextRun *run, const WebCoreTextStyle *style, const WebCoreTextGeometry *geom)
{
    assert(run);
    assert(style);
    assert(style->families);

    if (run->length == 0)
        return ;

    PangoLayout *layout = getPangoLayoutForGdkDrawable(m_context->realDrawable());
    setPangoAttributes(style, layout);

    gchar *utf8 = convertUniCharToUTF8(run->characters, 0, run->to);

    int x = geom->x;
    if (run->from > 0) {
        gchar *str_left = g_strdup(utf8);
        str_left = g_utf8_strncpy(str_left, utf8, run->from);
        PangoLayout *dummy_layout;
        int width;
        dummy_layout = pango_layout_copy(layout);
        pango_layout_set_text(dummy_layout, str_left, -1);
        pango_layout_get_pixel_size(dummy_layout, &width, 0);
        x += width;
        g_object_unref(dummy_layout);
        g_free(str_left);
    }

    gchar *str_from = g_utf8_offset_to_pointer(utf8, run->from);
    pango_layout_context_changed(layout);
    pango_layout_set_text(layout, str_from, -1);

    GdkGC *gc = gdk_gc_new(m_context->realDrawable());
    gdk_draw_layout(m_context->realDrawable(), gc, x, geom->y, layout);

    g_object_unref(gc);
    g_object_unref(layout);
    g_free(utf8);
}

void PangoTextRenderer::drawLineForCharacters(int x, int y, float yOffset, int width, GdkColor* color)
{
}


int PangoTextRenderer::pointToOffset(const WebCoreTextRun *run, const WebCoreTextStyle *style, int x, bool reversed, bool includePartialGlyphs)
{
    assert(run);
    assert(style);
    assert(style->families);

    if (run->length == 0)
        return 0;

    PangoLayout *layout = getPangoLayout();
    setPangoAttributes(style, layout);

    gchar *utf8 = convertUniCharToUTF8(run->characters, run->from, run->to);
    pango_layout_set_text(layout, utf8, -1);
    int index, trailing;
    pango_layout_xy_to_index(layout, x * PANGO_SCALE, 1, &index, &trailing);
    glong offset = g_utf8_pointer_to_offset(utf8, utf8+index);
    g_object_unref(layout);
    g_free(utf8);

    return offset;
}

void PangoTextRenderer::setPangoAttributes(const WebCoreTextStyle *style, PangoLayout *layout)
{
    PangoAttribute *attr;
    PangoAttrList *list;
    PangoContext *pango_context;
    PangoDirection direction;
    int i;
    const gchar *family;

    list = pango_attr_list_new();

    attr = pango_attr_fallback_new(TRUE);
    attr->end_index = G_MAXUINT;
    pango_attr_list_insert_before(list, attr);

    attr = pango_attr_font_desc_new(m_font->desc);
    attr->end_index = G_MAXUINT;
    pango_attr_list_insert_before(list, attr);

    for (i = 0; (family = style->families[i]); i++) {
        attr = pango_attr_family_new(family);
        attr->end_index = G_MAXUINT;
        pango_attr_list_insert_before(list, attr);
    }

    attr = pango_attr_foreground_new(style->textColor.red,
                                     style->textColor.green,
                                     style->textColor.blue);
    attr->end_index = G_MAXUINT;
    pango_attr_list_insert_before(list, attr);

    if (style->drawBackground) {
        attr = pango_attr_background_new(style->backgroundColor.red,
                                         style->backgroundColor.green,
                                         style->backgroundColor.blue);
        attr->end_index = G_MAXUINT;
        pango_attr_list_insert_before(list, attr);
    }

    attr = pango_attr_letter_spacing_new(style->letterSpacing * PANGO_SCALE);
    attr->end_index = G_MAXUINT;
    pango_attr_list_insert_before(list, attr);

    if (style->smallCaps) {
        attr = pango_attr_variant_new(PANGO_VARIANT_SMALL_CAPS);
        attr->end_index = G_MAXUINT;
        pango_attr_list_insert_before(list, attr);
    }

    pango_layout_set_attributes(layout, list);
    pango_attr_list_unref(list);

    pango_layout_set_auto_dir(layout, FALSE);

    pango_context = pango_layout_get_context(layout);
    direction = style->rtl ? PANGO_DIRECTION_RTL : PANGO_DIRECTION_LTR;
    pango_context_set_base_dir(pango_context, direction);
}

PangoLayout *PangoTextRenderer::getPangoLayoutForGdkDrawable(GdkDrawable *drawable)
{
    GdkScreen *screen = gdk_drawable_get_screen(drawable);
    PangoContext *pango_context = gdk_pango_context_get_for_screen(screen);
    PangoLayout *layout = pango_layout_new(pango_context);
    g_object_unref(pango_context);

    return layout;
}

PangoLayout *PangoTextRenderer::getPangoLayout()
{
    PangoContext *pango_context = gdk_pango_context_get();
    PangoLayout *layout = pango_layout_new(pango_context);
    g_object_unref(pango_context);

    return layout;
}

gchar *PangoTextRenderer::convertUniCharToUTF8(const UniChar *characters, int from, int to)
{
    const gunichar2* str = reinterpret_cast<const gunichar2*>(characters);
    gchar *utf8 = g_utf16_to_utf8(str, to, NULL, NULL, NULL);

    if (from > 0) {
        gchar *str_left = g_strdup(utf8);
        str_left = g_utf8_strncpy(str_left, utf8, from);
        g_free(utf8);
        utf8 = str_left;
    }

    gchar *pos = utf8;
    gint len = strlen(pos);
    GString *ret = g_string_new_len(NULL, len);

    // replace line break by space
    while (len > 0) {
        gint index, start;
        pango_find_paragraph_boundary(pos, len,
                                      &index, &start);
        g_string_append_len(ret, pos, index);
        if (index == start)
            break;
        g_string_append_c(ret, ' ');
        pos += start;
        len -= start;
    }
    return g_string_free(ret, FALSE);
}


/*
  vi:ts=4:nowrap:ai:expandtab
*/
