/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: f; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WebiEventListener.h"
#include "WebiRoot.h"
#include "rootimpl.h"
#include <assert.h>
#include "webi.h"
#include "webi-private.h"

#include "webkit-gtk/webkit-view-private.h"


WebiEventListener::WebiEventListener(WebiRoot* aroot)
    :root(aroot)
{
    strSz = 400;
    status = g_new0(gchar, strSz);
    loc = g_new0(gchar, strSz);
    pageTitle = g_new0(gchar, strSz);
    clearState();
}

WebiEventListener::~WebiEventListener()
{
    if (status)
        g_free(status);
    if (loc)
        g_free(loc);
    if (pageTitle)
        g_free(pageTitle);
}

void WebiEventListener::clearState()
{
    *status = *loc = *pageTitle = '\0';
}


void WebiEventListener::onSelectionChanged(FrameImpl* target)
{
    g_signal_emit_by_name(root->engine(), "selection");
}

static gboolean _utf8_copy_if_valid_else_set_empty(gchar* dst, const gchar* src, int dstsz)
{
    if (src &&g_utf8_validate(src, -1, NULL)) {
        glong sz = g_utf8_strlen(src, dstsz);
        g_utf8_strncpy(dst, src, sz+1);
        return TRUE;
    }
    *dst = '\0';
    return FALSE;
}

void WebiEventListener::onTitleChanged(FrameImpl* frame, const gchar* title)
{
    if (frame == root->mainFrame()) {
        _utf8_copy_if_valid_else_set_empty(pageTitle, title, strSz);
        g_signal_emit_by_name(root->engine(), "title");
    }
}

void WebiEventListener::onClientRedirectReceived(FrameImpl* frame,const gchar* to)
{
    frame = frame;
    to = to;
    /** not used*/
}

void WebiEventListener::onServerRedirected(FrameImpl* frame, const gchar* to)
{
    if (frame == root->mainFrame()) {
        g_signal_emit_by_name(root->engine(), "location");
    }
}

void WebiEventListener::onCommitLoad(FrameImpl* frame)
{
    if (frame == root->mainFrame()) {
        g_signal_emit_by_name(root->engine(), "location");
    }
}


bool WebiEventListener::onSetCookie(FrameImpl* target, const Cookie* cookie, const gchar* url)
{
    return root->internalOnSetCookie(cookie, url);
}

#if 0
gint
_convert_status(int status)
{
    /* map OSB status to Webi status*/
    return (gint) status;
}
#endif

void WebiEventListener::onFrameLoadStarted(FrameImpl* frame)
{
    if (frame == root->mainFrame()) {
        clearState();
        root->internalStatusStart();
        g_signal_emit_by_name(root->engine(), "load-start");
    }
}

void WebiEventListener::onFrameLoadFinished(FrameImpl* frame, int status)
{
    if (frame == root->mainFrame()) {
        root->internalStatusStop(status);
        g_signal_emit_by_name(root->engine(), "load-stop");
    }
}

void WebiEventListener::onResourceLoadStarted(FrameImpl* frame, const OSB::ResourceStatus * status)
{
    root->internalStatusStartItem(status);
}

void WebiEventListener::onResourceLoadHeaders(FrameImpl* frame, const OSB::ResourceStatus * status)
{
    root->internalStatusHeadersItem(status);
}

void WebiEventListener::onResourceLoadStatus(FrameImpl* frame, const OSB::ResourceStatus * status)
{
    root->internalStatusProgressItem(status);
}

void WebiEventListener::onResourceLoadFinished(FrameImpl* frame, const OSB::ResourceStatus * status)
{
    root->internalStatusStopItem(status);
}


void WebiEventListener::setWindowProperties(FrameImpl* frame, OSB::Features &features)
{
    WebiWindowProperties properties;

    properties.toolbarsVisible = features.areToolbarsVisible();
    properties.statusBarVisible = features.isStatusBarVisible();
    properties.scrollbarsVisible = features.areScrollbarsVisible();
    properties.isResizable = features.windowIsResizable();

    g_signal_emit_by_name(root->engine(), "set-window-properties", &properties);

}

void WebiEventListener::getWindowSize(FrameImpl* frame, GtkAllocation * size, bool isContentSize)
{
    WebiWindowSize type = (isContentSize ? WEBI_CONTENT_SIZE : WEBI_WINDOW_SIZE);
    g_signal_emit_by_name(root->engine(), "get-window-size", &type, size);
}

void WebiEventListener::setWindowSize(FrameImpl* frame, GtkRequisition * size, bool isContentSize)
{
    WebiWindowSize type = (isContentSize ? WEBI_CONTENT_SIZE : WEBI_WINDOW_SIZE);
    g_signal_emit_by_name(root->engine(), "set-window-size", &type, size);
}

bool WebiEventListener::authPanel(FrameImpl* frame, const gchar* url, const gchar *realm, gchar **username, gchar **password)
{
    WebiAuthArgs args = {0,};

    g_return_val_if_fail (frame != 0, false);
    g_return_val_if_fail (realm != 0, false);
    g_return_val_if_fail (username != 0, false);
    g_return_val_if_fail (password != 0, false);

    args.url = url;
    args.realm = realm;
    args.out_username = 0;
    args.out_password = 0;

    g_signal_emit_by_name (root->engine(), "req-auth-prompt", &args);

    /* propagate results to caller. results will be valid until next call to this method */
    if (args.out_username && args.out_password) {
        *username = args.out_username; /* no g_strdup() by api convention*/
        *password = args.out_password; /* no g_strdup() by api convention*/
    }

    return (args.out_ok_pressed == TRUE);
}

