/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: f; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WebiRoot.h"

#include "webi.h"
#include "rootimpl.h"
#include "frameimpl.h"
#include "Cookie.h"

WebiRoot::WebiRoot(RootImpl* root, Webi* webi)
    :listener(this)
    ,m_root(root) // owned
	,m_webkitView(0)
	,m_signalReceiver(G_OBJECT(webi))
    ,cookie(0)
    ,internal(false)
{
	root->setOldUIDelegate(&listener);
    root->setFrameLoadDelegate(&listener);
    root->setResourceLoadDelegate(&listener);
    status = g_new0(WebiLoadStatus, 1);
}

WebiRoot::WebiRoot(RootImpl* root, WebkitView* webkitView, WebkitViewWebiSignalStub* stub)
    :listener(this)
    ,m_root(root) // not owned
	,m_webkitView(webkitView)
	,m_signalReceiver(G_OBJECT(stub))
    ,cookie(0)
    ,internal(false)
{
	g_object_ref(m_signalReceiver);

    root->setOldUIDelegate(&listener);
    root->setFrameLoadDelegate(&listener);
    root->setResourceLoadDelegate(&listener);
    status = g_new0(WebiLoadStatus, 1);
}

WebiRoot::~WebiRoot()
{
    g_free(status);
    if (cookie) {
        g_free(cookie);
    }

	g_object_unref(m_signalReceiver);

	if (m_webkitView) {
		// called by signal_stub_init, dont delete m_root
		g_object_unref(m_webkitView);
	} else {
		// called by webi_init, delete
		delete(m_root);
	}
}
FrameImpl* WebiRoot::mainFrame()
{
	return m_root->mainFrame();
}

const WebiLoadStatus* WebiRoot::myStatus()
{
    return status;
}


const gchar* WebiRoot::myLocation()
{
    return m_root->mainFrame()->location();
}

const gchar* WebiRoot::myTitle()
{
    const gchar* l = listener.title();
    return l;
}

gboolean WebiRoot::findString(const gchar* text, gboolean case_sensitive, gboolean dir_down)
{
    return m_root->searchFor(text, dir_down, case_sensitive) ? TRUE:FALSE;
}

void WebiRoot::internalStatusStart()
{
    if (!internal) return;
    status->status = WEBI_LOADING_START;
    status->size = 0;
    status->received = 0;
    status->files = 0;
    status->filesWithSize = 0;
    status->ready = 0;

    g_signal_emit_by_name(m_signalReceiver, "status", status);
}

void WebiRoot::internalStatusStop(int statusCode)
{
    if (!internal) return;
    status->status = WEBI_LOADING_COMPLETE;
    status->statusCode = statusCode;

    g_signal_emit_by_name(m_signalReceiver, "status", status);
//    assert(status->ready == status->files);
}

void WebiRoot::internalStatusStartItem(const OSB::ResourceStatus * _status)
{
    if (!internal) return;

    status->status = WEBI_LOADING_START;
    status->files++;
    g_signal_emit_by_name(m_signalReceiver, "status", status);
}

void WebiRoot::internalStatusHeadersItem(const OSB::ResourceStatus * _status)
{
    if (!internal) return;

    status->status = WEBI_LOADING_START;
    if (_status->size()) {
        status->size += _status->size();
        status->totalSize += _status->size();
        status->filesWithSize++;
    }
    g_signal_emit_by_name(m_signalReceiver, "status", status);
}

void WebiRoot::internalStatusProgressItem(const OSB::ResourceStatus * _status)
{
    if (!internal) return;

    status->status = WEBI_LOADING;
    status->totalReceived += _status->lastReceived();
    // in case Content-Length -header is not present
    if (!_status->size()) {
        status->totalSize += _status->lastReceived();
    } else {
        status->received += _status->lastReceived();
    }
    g_signal_emit_by_name(m_signalReceiver, "status", status);
}

void WebiRoot::internalStatusStopItem(const OSB::ResourceStatus * _status)
{
    if (!internal) return;

    status->ready++;
    if (_status->error()) {
        // FIXME: handle errors as well.u
        status->status = WEBI_LOADING_ERROR;
    }

    g_signal_emit_by_name(m_signalReceiver, "status", status);
}

bool WebiRoot::internalOnSetCookie(const Cookie* _cookie, const gchar* url)
{
    if (!cookie) {
        cookie = g_new0(WebiCookie, 1);
    }

    cookie->name = _cookie->name();
    cookie->value = _cookie->value();
    cookie->comment = _cookie->comment();
    cookie->domain = _cookie->domain();
    cookie->path = _cookie->path();
    cookie->secure = _cookie->secure();
    cookie->version = _cookie->version();

    gboolean ret = FALSE;
    g_signal_emit_by_name(m_signalReceiver, "set-cookie", url, cookie,  &ret);

    // these need to be cleaned because _cookie could be deleted
    // after we exit from this function.
    cookie->name = 0;
    cookie->value = 0;
    cookie->comment = 0;
    cookie->domain = 0;
    cookie->path = 0;

    return ret;
}
