/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 * Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include "CurlRequest.h"
#include "CurlFactory.h"
#include "CookieStorage.h"

#include "gnet/uri.h"
#include "GLibHelpers.h"

#define HTTP_LOCATION "Location"
#define HTTP_CONTENT_TYPE "Content-Type"
#define HTTP_CONTENT_LENGTH "Content-Length"
#define HTTP_REFRESH "Http-Refresh"
#define HTTP_SET_COOKIE "Set-Cookie"
#define HTTP_WWW_AUTHENTICATE "WWW-Authenticate"

using namespace Loader;

extern "C" {
static size_t write_cb(void *ptr, size_t size, size_t nmemb, void *data)
{
	assert(data);
    CURL *handle = static_cast<CURL*>(data);
    CurlRequest *req = 0;
	int realsize = size * nmemb;

    curl_easy_getinfo(handle, CURLINFO_PRIVATE, &req);
    assert(req);

	req->dataFromCurl(handle, (char*)ptr, realsize);

	return realsize;
}

static size_t header_cb(void *ptr, size_t size, size_t nmemb, void* data)
{
    CURL *handle = static_cast<CURL*>(data);
    CurlRequest *req = 0;
	int realsize = size * nmemb;

    curl_easy_getinfo(handle, CURLINFO_PRIVATE, &req);
    assert(req);

	req->headerFromCurl(handle, (char*)ptr, realsize);
	return realsize;
}

}

CurlRequest::CurlRequest(CurlFactory* aParent,
						 RequestListener* aListener,
                         HTTPHeaderListener* headerListener,
                         HTTPAuthorizer* authorizer,
						 const gchar* aUrl,
						 const gchar* aCookies,
                         bool aIsSynchronous,
                         const gchar* postContentType,
                         GByteArray* postData)
    :m_parent(aParent)
    ,m_listener(aListener)
    ,m_headerListener(headerListener)
    ,m_authorizer(authorizer)
    ,m_cookies(0)
    ,m_handle(0)
    ,m_curlHeaders(0)
    ,m_postData(0)
    ,m_status(-1)
    ,m_isSynchronous(aIsSynchronous)
    ,m_headersEnd(false)
    ,m_finishSent(false)
    ,m_startSent(false)
    ,m_statusSent(false)
    ,m_authCredentials(0)
    ,m_authCancelled(false)
    ,m_authCancelHeaderList(0)
    ,m_reexecuteInProgress(false)
{
    assert(m_parent);

    if (postContentType && postData)
        setPostData(postContentType, postData);

    m_url = aUrl;
    m_cookies = aCookies;
}

CurlRequest::~CurlRequest()
{
    cleanup();

    if (m_postData)
		g_byte_array_free(m_postData, true);

    if (m_curlHeaders)
        curl_slist_free_all(m_curlHeaders);
}

void CurlRequest::cleanup()
{
    if (m_handle) {
        curl_easy_setopt(m_handle, CURLOPT_PRIVATE, 0);

		curl_multi_remove_handle(m_parent->multiHandle(), m_handle);
		curl_easy_cleanup(m_handle);
		m_handle = 0;
    }
    freeAuthCancelHeaderList();
}

void CurlRequest::queueReexecute()
{
    m_reexecuteInProgress = true;
    reset();
}
void CurlRequest::reset()
{
    assert(m_handle);
    freeAuthCancelHeaderList();

    if (m_curlHeaders) {
        curl_slist_free_all(m_curlHeaders);
        m_curlHeaders = 0;
    }

    if (m_postData) {
        g_byte_array_free(m_postData, true);
        m_postData = 0;
    }
}

void CurlRequest::setPostData(const gchar* contentType, GByteArray * data)
{
    assert((strcmp(contentType, "application/x-www-form-urlencoded") ||
            strncmp(contentType, "multipart/form-data", 19)));

    gchar* ctheader = g_strconcat("Content-Type: ", contentType, NULL);
    m_curlHeaders = curl_slist_append(m_curlHeaders, ctheader);
    g_free(ctheader);

    m_postData = g_byte_array_sized_new(data->len);
    memcpy(m_postData->data, data->data, data->len );
    m_postData->len = data->len;

}


void CurlRequest::execute()
{
    m_authCancelled = false;
    m_status = -1;
    m_reexecuteInProgress = false;

    if (!m_handle)
        m_handle = curl_easy_init();
    else
        curl_easy_reset(m_handle);

    curl_easy_setopt(m_handle, CURLOPT_PRIVATE, this);
    curl_easy_setopt(m_handle, CURLOPT_WRITEFUNCTION, ::write_cb);
    curl_easy_setopt(m_handle, CURLOPT_HEADERFUNCTION, ::header_cb);

    if (m_parent->proxy())
        curl_easy_setopt(m_handle, CURLOPT_PROXY, m_parent->proxy());

    if (m_agent)
        curl_easy_setopt(m_handle, CURLOPT_USERAGENT, m_agent.ptr());
    if (m_postData) {
        curl_easy_setopt(m_handle, CURLOPT_POSTFIELDS, m_postData->data);
        curl_easy_setopt(m_handle, CURLOPT_POSTFIELDSIZE, m_postData->len);
    }

    curl_easy_setopt(m_handle, CURLOPT_WRITEDATA, m_handle);
    curl_easy_setopt(m_handle, CURLOPT_WRITEHEADER, m_handle);

    // url ptr must remain valid through the request
    curl_easy_setopt(m_handle, CURLOPT_URL, m_url.ptr());

    m_cookies = WebCoreCookieAdapter::sharedAdapter()->cookiesForURL(m_url);
    if (m_cookies) {
        m_curlHeaders = curl_slist_append(m_curlHeaders, m_cookies.ptr());
    }

    if (m_curlHeaders)
        curl_easy_setopt(m_handle, CURLOPT_HTTPHEADER, m_curlHeaders);

    if (m_authCredentials) {
        if (g_ascii_strcasecmp(m_authScheme, "digest") == 0)
            curl_easy_setopt(m_handle, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
        else
            curl_easy_setopt(m_handle, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
		curl_easy_setopt(m_handle, CURLOPT_USERPWD, m_authCredentials.ptr());
    }

    if (!m_startSent) {
        m_startSent = true;
        m_listener->started(this);
    }

    CURLMcode ret = curl_multi_add_handle(m_parent->multiHandle(), m_handle);
    if (ret == CURLM_CALL_MULTI_PERFORM) {
		// don't call perform, because events must be async
		// timeout will occur and do curl_multi_perform
    } else if (ret == CURLM_OK) {

    } else {
		cleanup();
		m_listener->error(this);
    }

    if (m_isSynchronous) {
		while (!m_finishSent){
			m_parent->event();
		}
    }

}

void CurlRequest::stop()
{
    cleanup();

    // this will be deleted
    m_finishSent = true;
    m_listener->finished(this);
}

void CurlRequest::error(int curlerror)
{
#if DEBUG
    g_printerr("curl error while loading url:'%s' errorno:%d", m_url.ptr(), curlerror);
#endif

    m_listener->error(this);
}


void CurlRequest::dataFromCurl(CURL *origin, const char* data, int len)
{
    if (m_reexecuteInProgress)
        return;

    // we might get data to old handle, if  this requ has been connected
    // to a new handle due to auth

    if (origin != m_handle)
        return;

    if (m_status == HTTP_STATUS_AUTH_REQUIRED && !m_authCancelled)
        return;

    if (!m_headersEnd) {
        // replay the headers that may be put on hold b/c
        // it was not known if client wanted to cancel the auth
        GSList* hdrlst = m_authCancelHeaderList;
        while (hdrlst) {
            gchar* data = (gchar*) hdrlst->data;
            headerFromCurl(m_handle, data, strlen(data));
            hdrlst = hdrlst->next;
        }
        freeAuthCancelHeaderList();
		m_headersEnd = true;
    }

    if (!m_statusSent) {
        m_headerListener->setHTTPStatus(this, m_status);
        m_statusSent = true;
    }

    // if we are in middle of authenticating, do nothing.
    m_listener->data(this, data, len);
}

void CurlRequest::freeAuthCancelHeaderList()
{
    GSList* i = m_authCancelHeaderList;
    while (i) {
        g_free(i->data);
        i = i->next;
    }
    g_slist_free(m_authCancelHeaderList);
    m_authCancelHeaderList = 0;
}

void CurlRequest::rerequestWithCredentials(const gchar* user, const gchar* pass)
{
    if (user && pass) {
        m_authCancelled = false;
		m_authCredentials = g_strdup_printf("%s:%s", user, pass);
    }
}

void CurlRequest::headerFromCurl(CURL *origin, const char* data, int len)
{
    if (m_reexecuteInProgress) return;
    
    if (origin != m_handle)
        return;

    if (m_status == -1) {
        curl_easy_getinfo(m_handle, CURLINFO_RESPONSE_CODE, &m_status);
        assert(m_status != -1);
    }
    const gchar* colon = g_strstr_len(data, len, ":");

    if (!colon)
        return;

    // these are freed
    int keylen = colon-data;
    gchar* key = g_strstrip(g_strndup(data, keylen ));
    gchar* value = g_strstrip(g_strndup(colon+1, len - keylen -1));

    if (m_status == HTTP_STATUS_AUTH_REQUIRED && !m_authCancelled) {
        // we should prompt user for authentication credentials

        // extract authentication header or record other headers
        // for later playback

        if (!g_ascii_strcasecmp(key, HTTP_WWW_AUTHENTICATE) && value) {
            m_authScheme = 0;
            m_authCredentials = 0;

            m_authCancelled = true;

			gchar* realm = g_strstrip(strstr(value, "realm="));
            if (realm) {
                gchar* authScheme = g_strndup(value, realm - value);
                gchar* authRealm = g_strdup(realm+6);

                // this call should set m_authCancelled if auth is wanted 
                m_authorizer->HTTPAuthRequest(m_url, g_strstrip(authRealm), this);

                g_free (authRealm);

                if (!m_authCancelled)
                    m_authScheme = authScheme;
                else
                    g_free(authScheme);
            }
		} else {
            // in case HTTP_AUTH is requred, but the user cancels
            // the auth, we need to use all the headers
            m_authCancelHeaderList =
               g_slist_append(m_authCancelHeaderList, g_strndup(data, len));
        }

        g_free(key);
        g_free(value);
        return;
    }

    if (!m_statusSent) {
        m_headerListener->setHTTPStatus(this, m_status);
        m_statusSent = true;
    }

    bool headerHandled = false;

    if (!g_ascii_strcasecmp(key, HTTP_LOCATION)) {
        /* http 1.1 (or 1.0?) doesn't allow relative Location:
         * addresses, but we still get those. modify them here
         * as absolute  based on request uri
         */
        GURI *url = gnet_uri_new(m_url);
        if (url) {
            GURI *newurl = gnet_uri_new_relative(url, value);
            if (newurl) {
                m_url = gnet_uri_get_string(newurl);

                gnet_uri_delete(newurl);
                m_headerListener->setLocation(this, m_url);
                queueReexecute();
                headerHandled = true;
            }
            gnet_uri_delete(url);
        }
    } else if (!g_ascii_strcasecmp(key, HTTP_CONTENT_TYPE)) {

        gchar* encoding = 0;
        gchar* contentType = 0;

        gchar **chunks = g_strsplit(value, ";", 2);
        assert(chunks);

        int i = 0;
        while(chunks[i]) {
            gchar* charset = strcasestr(chunks[i], "charset=");
            if (charset) {
                // assume "CONTENT-TYPE; charset=CHARSET-NAME" or "charset=CHARSET-NAME"
                if (!encoding) {
                    encoding = g_strstrip(g_strdup(charset+8));
                    g_free(chunks[i]);
                }
            } else {
                // assume "CONTENT-TYPE" or "CONTENT-TYPE; CHARSET-NAME"
                if (!contentType)
                    contentType = chunks[i];
                else if (!encoding)
                    encoding = chunks[i];
            }
            ++i;
        }
        g_free(chunks);

        if (contentType || encoding) {
            m_headerListener->setContentType(this, contentType, encoding);
            headerHandled = true;
            if (contentType)
                g_free(contentType);
            if (encoding)
                g_free(encoding);
        }

    } else if (!g_ascii_strcasecmp(key, HTTP_CONTENT_LENGTH)) {

        gchar* endptr = 0;
        long length = strtol(value, &endptr, 10);
        if (*value != '\0' && endptr && *endptr=='\0') {
            m_headerListener->setContentLength(this, length);
            headerHandled = true;
        }
    } else if (!g_ascii_strcasecmp(key, HTTP_REFRESH)) {
        m_headerListener->setRefresh(this, value);
        headerHandled = true;
    } else if (!g_ascii_strcasecmp(key, HTTP_SET_COOKIE)) {
        m_headerListener->setCookie(this, value);
        headerHandled = true;
    }

    if (!headerHandled)
        m_headerListener->setUnknownHTTPHeader(this, key, value);


    g_free(key);
    g_free(value);
}

void CurlRequest::finish()
{
    if (!m_reexecuteInProgress){
        cleanup();

        // might delete the request!
        m_finishSent = true;
        m_listener->finished(this);
    }
}

