/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (c) 2004 Nokia. All rights reserved.
 * Copyright (c) 2007 Kimmo Kinnunen. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * Neither the name of Nokia nor the names of its contributors may be
 * used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "webkit-view.h"
#include "webkit-view-webi-extensions.h"
#include "webkit-view-private.h"

#include "webi-marshal.h"

#include "rootimpl.h"
#include "frameimpl.h"

#include "WebiRoot.h" // for signal stub support

static void webkit_view_finalize (GObject * self) G_GNUC_UNUSED;
static void webkit_view_realize (GtkWidget * widget) G_GNUC_UNUSED;
static void webkit_view_map (GtkWidget * widget) G_GNUC_UNUSED;
static void webkit_view_unmap (GtkWidget * widget) G_GNUC_UNUSED;
static gboolean webkit_view_expose_event (GtkWidget * widget, GdkEventExpose * event) G_GNUC_UNUSED;
static void webkit_view_size_allocate (GtkWidget * widget, GtkAllocation * allocation) G_GNUC_UNUSED;

G_DEFINE_TYPE(WebkitView, webkit_view, GTK_TYPE_BIN);

#define WEBKIT_VIEW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_VIEW, WebkitViewPrivate))
#define SELF_PRIV(self) WebkitViewPrivate* priv = WEBKIT_VIEW_GET_PRIVATE(self)
static void webkit_view_finalize(GObject *self);

static void webkit_view_class_init (WebkitViewClass *klass)
{
	GObjectClass *g_object_class = (GObjectClass*) klass;
	GtkWidgetClass *gtk_widget_class = (GtkWidgetClass *)klass;

	g_type_class_add_private(klass, sizeof(WebkitViewPrivate));
	g_object_class->finalize = webkit_view_finalize;
	gtk_widget_class->realize = webkit_view_realize;
	gtk_widget_class->map = webkit_view_map;
	gtk_widget_class->unmap = webkit_view_unmap;
	gtk_widget_class->expose_event = webkit_view_expose_event;
	gtk_widget_class->size_allocate = webkit_view_size_allocate;
}

static void webkit_view_finalize(GObject *objself)
{
	WebkitView *self = WEBKIT_VIEW (objself);
	SELF_PRIV(objself);

	if (priv->location) g_free (priv->location);
	if (priv->title) g_free (priv->title);
	if (priv->js_input_text) g_free (priv->js_input_text);
	delete(priv->signalstubroot);
#define PARENT_HANDLER(___self) \
    { if(G_OBJECT_CLASS(webkit_view_parent_class)->finalize) \
        (* G_OBJECT_CLASS(webkit_view_parent_class)->finalize)(___self); }

	PARENT_HANDLER(G_OBJECT (self));
	return;
#undef PARENT_HANDLER
}

static void
webkit_view_realize (GtkWidget * widget)
#define PARENT_HANDLER(___widget) \
    { if(GTK_WIDGET_CLASS(webkit_view_parent_class)->realize) \
        (* GTK_WIDGET_CLASS(webkit_view_parent_class)->realize)(___widget); }
{


    GdkWindowAttr   attributes;
    gint            attributes_mask;

    g_return_if_fail(widget != NULL);

    GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);

    attributes.window_type = GDK_WINDOW_CHILD;
    attributes.x = widget->allocation.x;
    attributes.y = widget->allocation.y;
    attributes.width = widget->allocation.width;
    attributes.height = widget->allocation.height;
    attributes.wclass = GDK_INPUT_OUTPUT;
    attributes.visual = gtk_widget_get_visual (widget);
    attributes.colormap = gtk_widget_get_colormap (widget);
    attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;

    attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

    widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
                                     &attributes, attributes_mask);
    gdk_window_set_user_data (widget->window, widget);

    widget->style = gtk_style_attach(widget->style, widget->window);

}
#undef PARENT_HANDLER

static void
webkit_view_map (GtkWidget * widget)
#define PARENT_HANDLER(___widget) \
    { if(GTK_WIDGET_CLASS(webkit_view_parent_class)->map) \
        (* GTK_WIDGET_CLASS(webkit_view_parent_class)->map)(___widget); }
{

    GtkWidget* child;
    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBKIT_IS_VIEW (widget));

    child = GTK_BIN(widget)->child;

    GTK_WIDGET_SET_FLAGS(widget, GTK_MAPPED);

    //NEEDED?
    if (GTK_WIDGET_VISIBLE(child))
    {
        if (!GTK_WIDGET_MAPPED (child)){
            gtk_widget_map(child);
        }
    }
    gdk_window_show(widget->window);
}
#undef PARENT_HANDLER

static void
webkit_view_unmap (GtkWidget * widget)
{

    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBKIT_IS_VIEW (widget));

    GTK_WIDGET_UNSET_FLAGS(widget, GTK_MAPPED);

    gdk_window_hide(widget->window);
}

static gboolean
webkit_view_expose_event (GtkWidget * widget, GdkEventExpose * event)
#define PARENT_HANDLER(___widget,___event) \
    ((GTK_WIDGET_CLASS(webkit_view_parent_class)->expose_event)? \
        (* GTK_WIDGET_CLASS(webkit_view_parent_class)->expose_event)(___widget,___event): \
        ((gboolean )0))
{

    return PARENT_HANDLER(widget,event);
}

#undef PARENT_HANDLER

static void
webkit_view_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
#define PARENT_HANDLER(___widget,___allocation) \
    { if(GTK_WIDGET_CLASS(webkit_view_parent_class)->size_allocate) \
        (* GTK_WIDGET_CLASS(webkit_view_parent_class)->size_allocate)(___widget,___allocation); }
{

    GtkWidget* child;
    g_return_if_fail(widget != NULL);
    g_return_if_fail(WEBKIT_IS_VIEW (widget));

    widget->allocation = *allocation;

    // NEEDED? Not in Mozilla
    child = GTK_BIN(widget)->child;
    if (child){
        GtkAllocation a;
        GtkRequisition r;
        gtk_widget_size_request(child, &r);

        a.x = 0;
        a.y = 0;
        /* a.height = r.height; */
        /*  a.width = r.width; */

        a.width = allocation->width;
        a.height = allocation->height;

        gtk_widget_size_allocate(child, &a);
    }

    if (GTK_WIDGET_REALIZED(widget))
    {
        gdk_window_move_resize(widget->window,
                               allocation->x, allocation->y,
                               allocation->width, allocation->height);
    }
}
#undef PARENT_HANDLER


static void on_webkit_view_destroy(GtkObject *object, gpointer user_data)

{
	WebkitView* self = WEBKIT_VIEW (object);
	SELF_PRIV(self);

	/* GtkBin has removed child from widget structure and decreased its reference count */
	/* destruct structures, which leads to destruction of child and all inner objects */

	if (priv->root) delete (priv->root);
	priv->root = 0;

}

static void webkit_view_init(WebkitView *self)
{
	SELF_PRIV(self);

	GTK_WIDGET_SET_FLAGS (self, GTK_CAN_FOCUS);

	OSB::Preferences *settings;
	RootImpl* root = new RootImpl(self);
	priv->root = root;
	priv->signalstubroot = 0;
	priv->child = priv->root->mainFrame()->widget();
	
	settings = priv->root->preferences();

	settings->setWillLoadImagesAutomatically(true);
	settings->setJavaScriptEnabled(true);
	settings->setJavaEnabled(false);
	settings->setPluginsEnabled(false);
	settings->setMinimumFontSize(6);
	settings->setDefaultFontSize(14);
	settings->setDefaultFixedFontSize(10);
	settings->setDefaultTextEncoding("ISO-8859-1");
	settings->setSerifFontFamily("serif");
	settings->setSansSerifFontFamily("sans");
	settings->setFixedFontFamily("fixed");
	settings->setStandardFontFamily("times");
	settings->setCursiveFontFamily("times");
	settings->setFantasyFontFamily("lucidabright");
	root->setDeviceType(OSB::DeviceScreen);

	GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET (self), GTK_NO_WINDOW);

	gtk_container_add (GTK_CONTAINER (self), priv->child);

	/* to destruct structures (which destructs priv->child)*/
	g_signal_connect (G_OBJECT (self),
			  "destroy",
			  G_CALLBACK (on_webkit_view_destroy),
			  self);

	return;
}


GtkWidget* webkit_view_new(const gchar* frame_name, const gchar* group_name)
{
/* webkit_view needs threading, because nrcit might use libsoup, which uses
 * threads
 */
	if (!g_thread_supported ()) g_thread_init (NULL);

	WebkitView* self = WEBKIT_VIEW(g_object_new (WEBKIT_TYPE_VIEW,  NULL));
	SELF_PRIV(self);

	priv->root->setGroup(frame_name);
	priv->root->mainFrame()->setFrameName(frame_name);


	return (GtkWidget*) self;
}

WebkitFrame* webkit_view_get_main_frame(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);

	return priv->root->mainFrame()->api();
}

const gchar* webkit_view_get_group_name(WebkitView* self)
{
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);
	
	RootImpl* root = priv->root;
	return root->group();
}

void webkit_view_set_group_name(WebkitView* self, const gchar* group_name)
{
	g_return_if_fail (self != NULL);
	g_return_if_fail (WEBKIT_IS_VIEW (self));
	SELF_PRIV(self);

	RootImpl* root = priv->root;
	root->setGroup(group_name);
}

gboolean webkit_view_go_back(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), FALSE);
	SELF_PRIV (self);

	if (priv->root->canGoBack()) {
		priv->root->goBack();
		return TRUE;
	}
	return FALSE;
}

gboolean webkit_view_go_forward(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), FALSE);

	SELF_PRIV (self);
	if (priv->root->canGoForward()) {
		priv->root->goForward();
		return TRUE;
	}
	return FALSE;
}

void webkit_view_set_text_multiplier (WebkitView* self, gfloat multiplier)
{
	g_return_if_fail (WEBKIT_IS_VIEW (self));
	SELF_PRIV (self);
	priv->root->setTextSizeMultiplier(multiplier);
}

gfloat webkit_view_get_text_multiplier (WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), 0.0);
	SELF_PRIV (self);

	return priv->root->textSizeMultiplier();
}

gboolean webkit_view_can_go_back (WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), FALSE);
	SELF_PRIV (self);
	return priv->root->canGoBack();
}

gboolean webkit_view_can_go_forward (WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), FALSE);
	SELF_PRIV (self);
	return priv->root->canGoForward();
}

void webkit_view_set_application_name_for_user_agent(WebkitView* self, const gchar* application_name)
{
	g_return_if_fail (WEBKIT_IS_VIEW (self));
	assert(0);
}

const gchar* webkit_view_get_application_name_for_user_agent(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	assert(0);
	return NULL;
}

void webkit_view_set_custom_user_agent(WebkitView* self, const gchar* user_agent_string)
{
	g_return_if_fail (WEBKIT_IS_VIEW (self));
	assert(0);
}

const gchar* webkit_view_get_custom_user_agent(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	assert(0);
	return NULL;
}

gboolean webkit_view_search_for (WebkitView* self, const gchar* text, gboolean direction_forward, gboolean case_sensitive, gboolean /*wrap*/)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), FALSE);
	SELF_PRIV(self);

	return priv->root->searchFor(text, direction_forward, case_sensitive);
}

gchar* webkit_view_get_current_selection_as_text(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);
	return priv->root->selectedText();
}

const gchar* webkit_view_get_main_frame_url(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);

	return priv->root->mainFrame()->location();
}

const gchar* webkit_view_get_main_frame_title(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);

	return priv->root->mainFrame()->title();
}

/* extensions to the api! */

RootImpl* webkit_view_get_root(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);
	SELF_PRIV(self);
	return priv->root;
}

void webkit_view_webi_set_default_external_content_delegate (WebkitView* self, WebiExternalContentDelegate* delegate)
{
    g_return_if_fail (WEBKIT_IS_VIEW (self));
    SELF_PRIV(self);
    priv->root->setDefaultExternalContentDelegate(delegate);

}

WebiExternalContentDelegate* webkit_view_webi_get_default_external_content_delegate (WebkitView* self)
{
    g_return_val_if_fail (WEBKIT_IS_VIEW (self), NULL);

    SELF_PRIV(self);
    return priv->root->getDefaultExternalContentDelegate();

}

/** deprecated!!!
 * Gets the engine user agent string.
 * The returned string can be used to construct browser-specific
 * user-agent strings
 */
const gchar* webkit_view_webi_get_engine_user_agent_string ()
{
	return BridgeImpl::engineUserAgentString();
}


void webkit_view_webi_set_settings(WebkitView* self, const WebiSettings* settings)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBKIT_IS_VIEW (self));
    SELF_PRIV(self);

    OSB::Preferences *s = priv->root->preferences();

    s->setWillLoadImagesAutomatically (settings->autoload_images);
    s->setJavaScriptEnabled (settings->javascript_enabled);
    s->setJavaEnabled (settings->java_enabled);
    s->setPluginsEnabled (settings->plugins_enabled);
    s->setMinimumFontSize (settings->minimum_font_size);
    s->setDefaultFontSize (settings->default_font_size);
    s->setDefaultFixedFontSize (settings->default_fixed_font_size);

#define CALL_IF_SET(inst, method, var) if ((var)) (inst)->method (var);

    CALL_IF_SET(s, setDefaultTextEncoding, settings->default_text_encoding);
    CALL_IF_SET(s, setSerifFontFamily, settings->serif_font_family);
    CALL_IF_SET(s, setSansSerifFontFamily, settings->sans_serif_font_family);
    CALL_IF_SET(s, setFixedFontFamily, settings->fixed_font_family);
    CALL_IF_SET(s, setStandardFontFamily, settings->standard_font_family);
    CALL_IF_SET(s, setCursiveFontFamily, settings->cursive_font_family);
    CALL_IF_SET(s, setFantasyFontFamily, settings->fantasy_font_family);
#undef CALL_IF_SET

    OSB::ProtocolDelegate* httpprov = priv->root->protocolDelegateForURL("http://");
    if (httpprov) {
        httpprov->setProxy(settings->http_proxy);
    }

    if (settings->user_agent_string) {
        priv->root->setUserAgentString(settings->user_agent_string);
    }
}

void webkit_view_webi_set_emit_internal_status (WebkitView* self, gboolean flag)
{
    g_return_if_fail (self != NULL);
    g_return_if_fail (WEBKIT_IS_VIEW (self));
    SELF_PRIV(self);
    priv->signalstubroot->setEmitInternal(flag == TRUE);
}


WebkitViewWebiSignalStub* webkit_view_webi_get_signal_stub(WebkitView* self)
{
	g_return_val_if_fail (WEBKIT_IS_VIEW_WEBI_SIGNAL_STUB (self), NULL);
	WebkitViewWebiSignalStub* real_self =  WEBKIT_VIEW_WEBI_SIGNAL_STUB (self);
	SELF_PRIV(self);
	
	if (!priv->signalstubroot)
		priv->signalstubroot = new WebiRoot(priv->root, self, real_self);

	return real_self;
}

/* implementation of the signalstub */

static void webkit_view_webi_signal_stub_finalize(GObject* self);

G_DEFINE_TYPE (WebkitViewWebiSignalStub, webkit_view_webi_signal_stub, WEBKIT_TYPE_VIEW);

enum {
    STATUS_TEXT_SIGNAL,
    STATUS_SIGNAL,
    LOCATION_SIGNAL,
    TITLE_SIGNAL,
    PROGRESS_SIGNAL,
    REQ_NEW_WINDOW,
    SHOW_WINDOW,
    CLOSE_WINDOW,
    SET_WINDOW_PROPERTIES,
    GET_WINDOW_SIZE,
    SET_WINDOW_SIZE,
    GET_DEVICE_TYPE,
    SET_DEVICE_TYPE,
    LOAD_START,
    LOAD_STOP,
    LOAD_ITEM_START,
    LOAD_ITEM_PROGRESS,
    LOAD_ITEM_STOP,
    SET_COOKIE,
    REQ_JS_PROMPT,
    REQ_AUTH_PROMPT,
    MOUSE_OVER,
    SELECTION,
    LAST_SIGNAL
};

static guint object_signals[LAST_SIGNAL] = {0};

static void webkit_view_webi_signal_stub_class_init (WebkitViewWebiSignalStubClass * c)
{
    GObjectClass *g_object_class = (GObjectClass*) c;

    c->status = NULL;
    c->status_text = NULL;
    c->location = NULL;
    c->title = NULL;
    c->req_new_window = NULL;
    c->show_window = NULL;
    c->close_window = NULL;
    c->set_window_properties = NULL;
    c->get_window_size = NULL;
    c->set_window_size = NULL;
    c->get_device_type = NULL;
    c->set_device_type = NULL;
    c->load_start = NULL;
    c->load_stop = NULL;
    c->req_js_prompt = NULL;
    c->req_auth_prompt = NULL;

    object_signals[STATUS_TEXT_SIGNAL] =
        g_signal_new ("status_text",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, status_text),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[STATUS_SIGNAL] =
        g_signal_new ("status",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, status),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[LOCATION_SIGNAL] =
        g_signal_new ("location",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, location),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[TITLE_SIGNAL] =
        g_signal_new ("title",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, title),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[REQ_NEW_WINDOW] =
        g_signal_new ("req-new-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, req_new_window),
                      NULL, NULL,
                      webi_marshal_POINTER__POINTER,
                      G_TYPE_POINTER, 1, G_TYPE_POINTER);
    object_signals[SHOW_WINDOW] =
        g_signal_new ("show-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, show_window),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[CLOSE_WINDOW] =
        g_signal_new ("close-window",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, close_window),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
    object_signals[SET_WINDOW_PROPERTIES] =
        g_signal_new ("set-window-properties",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, set_window_properties),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1, G_TYPE_POINTER);
    object_signals[GET_WINDOW_SIZE] =
        g_signal_new ("get-window-size",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, get_window_size),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER,
                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
    object_signals[SET_WINDOW_SIZE] =
        g_signal_new ("set-window-size",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, set_window_size),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER,
                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
    object_signals[GET_DEVICE_TYPE] =
        g_signal_new ("get-device-type",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, get_device_type),
                      NULL, NULL,
                      webi_marshal_INT__VOID,
                      G_TYPE_INT, 0);
    object_signals[SET_DEVICE_TYPE] =
        g_signal_new ("set-device-type",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, set_device_type),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__INT,
                      G_TYPE_NONE, 1, G_TYPE_INT);

    object_signals[LOAD_START] =
        g_signal_new ("load-start",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, load_start),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    object_signals[LOAD_STOP] =
        g_signal_new ("load-stop",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, load_stop),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    object_signals[SET_COOKIE] =
        g_signal_new ("set-cookie",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, set_cookie),
                      NULL, NULL,
                      webi_marshal_BOOLEAN__POINTER_POINTER,
                      G_TYPE_BOOLEAN, 2,
                      G_TYPE_POINTER, G_TYPE_POINTER);

    object_signals[REQ_JS_PROMPT] =
        g_signal_new ("req-js-prompt",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, req_js_prompt),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[REQ_AUTH_PROMPT] =
        g_signal_new ("req-auth-prompt",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, req_auth_prompt),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE, 1,
                      G_TYPE_POINTER);
    object_signals[MOUSE_OVER] =
        g_signal_new ("mouse-over",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, mouse_over),
                      NULL, NULL,
                      webi_marshal_VOID__POINTER_POINTER_POINTER_POINTER,
                      G_TYPE_NONE, 4,
                      G_TYPE_POINTER,G_TYPE_POINTER,G_TYPE_POINTER,G_TYPE_POINTER );
    object_signals[SELECTION] =
        g_signal_new ("selection",
                      G_TYPE_FROM_CLASS (g_object_class),
                      (GSignalFlags)(G_SIGNAL_RUN_LAST),
                      G_STRUCT_OFFSET (WebkitViewWebiSignalStubClass, selection),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);

    g_object_class->finalize = webkit_view_webi_signal_stub_finalize;

    return;
}

static void webkit_view_webi_signal_stub_init (WebkitViewWebiSignalStub* self)
{

}

static void webkit_view_webi_signal_stub_finalize(GObject* self)
{
#define PARENT_HANDLER(___self) \
    { if(G_OBJECT_CLASS(webkit_view_webi_signal_stub_parent_class)->finalize) \
        (* G_OBJECT_CLASS(webkit_view_webi_signal_stub_parent_class)->finalize)(___self); }

	PARENT_HANDLER(G_OBJECT (self));
	return;
#undef PARENT_HANDLER
}

#include "deprecated-signal-sender.h"

GtkWidget* webkit_view_webi_new_with_signal_stub(const gchar* frame_name, const gchar* group_name)
{
	if (!g_thread_supported ()) g_thread_init (NULL);

	WebkitView* self = WEBKIT_VIEW(g_object_new (WEBKIT_TYPE_VIEW_WEBI_SIGNAL_STUB,  NULL));
	SELF_PRIV(self);

	priv->root->setGroup(frame_name);
	priv->root->mainFrame()->setFrameName(frame_name);
	priv->root->setDefaultUIDelegate(deprecated_signal_sender_new(G_OBJECT(self)));

	return (GtkWidget*) self;
}
